==> libsambox-java_1.1.55.orig.tar.xz: sambox-1.1.55/src/main/java/org/sejda/sambox/pdmodel/interactive/documentnavigation/outline/PDOutlineItemIterator.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.sejda.sambox.pdmodel.interactive.documentnavigation.outline;

import java.util.Iterator;

/**
 * Iterator over the linked list of {@link PDOutlineItem} siblings.
 * 
 * @author Andrea Vacondio
 *
 */
class PDOutlineItemIterator implements Iterator<PDOutlineItem>
{
    private PDOutlineItem currentItem;
    private final PDOutlineItem startingItem;

    PDOutlineItemIterator(PDOutlineItem startingItem)
    {
        this.startingItem = startingItem;
    }

    @Override
    public boolean hasNext()
    {
        return startingItem != null
                && (currentItem == null || (currentItem.getNextSibling() != null && !startingItem
                        .equals(currentItem.getNextSibling())));
    }

    @Override
    public PDOutlineItem next()
    {
        if (currentItem == null)
        {
            currentItem = startingItem;
        }
        else
        {
            currentItem = currentItem.getNextSibling();
        }
        return currentItem;
    }

    @Override
    public void remove()
    {
        throw new UnsupportedOperationException();
    }
}

==> libsambox-java_2.2.11.orig.tar.xz: sambox-2.2.11/src/main/java/org/sejda/sambox/pdmodel/interactive/documentnavigation/outline/PDOutlineItemIterator.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.sejda.sambox.pdmodel.interactive.documentnavigation.outline;

import java.util.HashSet;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator over the linked list of {@link PDOutlineItem} siblings.
 * 
 * @author Andrea Vacondio
 *
 */
class PDOutlineItemIterator implements Iterator<PDOutlineItem>
{
    private PDOutlineItem currentItem;
    private final PDOutlineItem startingItem;
    private final HashSet<PDOutlineItem> visited = new HashSet<>();

    PDOutlineItemIterator(PDOutlineItem startingItem)
    {
        this.startingItem = startingItem;
    }

    @Override
    public boolean hasNext()
    {
        if (startingItem == null)
        {
            return false;
        }
        
        if(currentItem == null)
        {
            return true;
        }
        
        PDOutlineItem next = currentItem.getNextSibling();
        if (next != null)
        {
           if (startingItem.equals(currentItem.getNextSibling()) || visited.contains(next))
           {
               // infinite loop detected
               return false;
           } 
           else 
           {
               return true;
           }
        }
        
        return false;
    }

    @Override
    public PDOutlineItem next()
    {
        if (!hasNext())
        {
            throw new NoSuchElementException();
        }
        if (currentItem == null)
        {
            currentItem = startingItem;
        }
        else
        {
            currentItem = currentItem.getNextSibling();
        }
        
        if(visited.contains(currentItem))
        {
            throw new NoSuchElementException();
        }
        
        visited.add(currentItem);
        return currentItem;
    }

    @Override
    public void remove()
    {
        throw new UnsupportedOperationException();
    }
}
